//
//  DetailViewController.m
//  uPnp
//
//  Created by rk on 2/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "DetailViewController.h"
#import "RootViewController.h"


@interface DetailViewController ()
@property (nonatomic, retain) UIPopoverController *popoverController;
- (void)configureView;
@end



@implementation DetailViewController

@synthesize toolbar, popoverController, detailItem, detailDescriptionLabel, deviceInfoView, detailTextView;
@synthesize nameLabel, manufacturerLabel, modelLabel, typeLabel, udnLabel;

#pragma mark -
#pragma mark Managing the detail item

/*
 When setting the detail item, update the view and dismiss the popover controller if it's showing.
 */
- (void)setDetailItem:(id)newDetailItem {
    if (detailItem != newDetailItem) {
        [detailItem release];
        detailItem = [newDetailItem retain];
        
        // Update the view.
        [self configureView];
    }

    if (self.popoverController != nil) {
        [self.popoverController dismissPopoverAnimated:YES];
    }        
}


- (void) clearView {
	detailDescriptionLabel.text = nil;
	detailTextView.text = nil;
	
	nameLabel.text = nil;
	manufacturerLabel.text = nil;
	modelLabel.text = nil;
	typeLabel.text = nil;
	udnLabel.text = nil;
}


- (void)configureView {
    // Update the user interface for the detail item.
    
	[self clearView];
	
	if ([detailItem respondsToSelector:@selector(_deviceInfo)]) {
		nameLabel.text = [[detailItem _deviceInfo] friendlyName];
		manufacturerLabel.text = [[detailItem _deviceInfo] manufacturer];
		modelLabel.text = [[detailItem _deviceInfo] modelName];
		typeLabel.text = [[detailItem _deviceInfo] deviceTypeSchema];
		udnLabel.text = [[detailItem _deviceInfo] UDN];
	}
	
	if([detailItem respondsToSelector:@selector(_requestResponse)]) {
		detailDescriptionLabel.text = [detailItem _requestResponse];
		detailTextView.text = [detailItem _requestResponse];
	} else {
		detailDescriptionLabel.text = [detailItem description]; 
	}		
}



#pragma mark -
#pragma mark Split view support

- (void)splitViewController: (UISplitViewController*)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem*)barButtonItem forPopoverController: (UIPopoverController*)pc {
    
    barButtonItem.title = @"Devices";
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items insertObject:barButtonItem atIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = pc;
}


// Called when the view is shown again in the split view, invalidating the button and popover controller.
- (void)splitViewController: (UISplitViewController*)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem {
    
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items removeObjectAtIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = nil;
}


#pragma mark -
#pragma mark Rotation support

// Ensure that the view controller supports rotation and that the split view can therefore show in both portrait and landscape.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}


#pragma mark -
#pragma mark View lifecycle

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
//- (void)viewDidLoad {
//    [super viewDidLoad];
//		
//	//deviceInfoViewController = [[DeviceInfoViewController alloc] initWithNibName:@"UpnpDeviceInfoView" bundle:nil ];
//	//[deviceInfoViewController setView:deviceInfoView];
//	deviceInfoView = [deviceInfoViewController view];
//	[deviceInfoView setNeedsDisplay];
//	 
//}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/

- (void)viewDidUnload {
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    self.popoverController = nil;
}


#pragma mark -
#pragma mark Memory management

/*
- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}
*/

- (void)dealloc {
    [popoverController release];
    [toolbar release];
    
    [detailItem release];
    [detailDescriptionLabel release];
    [super dealloc];
}

@end
