//
//  RootViewController.m
//  uPnp
//
//  Created by rk on 2/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "RootViewController.h"
#import "DetailViewController.h"


@implementation RootViewController

@synthesize _upnpManager;
@synthesize _detailViewController;
@synthesize _devices;

#pragma mark -
#pragma mark Discover Devices

- (IBAction) discoverDevices : (id)sender{
	
	if (_upnpManager == nil) {
		_upnpManager = [[[UpnpManager alloc] init] retain];
	}
	
	if (_hud != nil) return; // only allow one discoverDevices request at a time.
	
	_hud = [[MBProgressHUD alloc] initWithView:_detailViewController.view];
	// Add HUD to screen
    [_detailViewController.view addSubview:_hud];
	
    // Register for HUD callbacks so we can remove it from the window at the right time
    _hud.delegate = self;
	
	
	// Clear down the list of devices 
	if (_devices != nil) {
		[_devices release];
	}
	_devices = [[NSMutableArray array] retain];
	//[_devices retain];
	_discoveringDevices = true;
	[self.tableView reloadData];
	
	// Show the HUD while the provided method executes in a new thread
    [_hud showWhileExecuting:@selector(doDiscoverDevices) onTarget:self withObject:nil animated:YES];
	
}

// This method gets executed in a new thread, not main thread
- (void) doDiscoverDevices {
	_discoveringDevices = true;
	@synchronized (self) {
		_temporaryDevices = [_upnpManager discoverDevices];
	}
	_discoveringDevices = false;
}
	

// This method gets executed in main thread
- (void)discoverDevicesCompletion {
    NSIndexPath *currentSelection = [self.tableView indexPathForSelectedRow];
    if (currentSelection != nil) {
        [self.tableView deselectRowAtIndexPath:currentSelection animated:NO];
    }    
	
	if (_devices != nil) {
		[_devices release];
	}
	
	@synchronized (self){
	_devices = _temporaryDevices;
	_temporaryDevices = nil;
	}
	
	UpnpDevice *device;
	for (device in _devices) {
		[device fetchDeviceInfo];
	}
	
	//[_upnpManager fetchDeviceInfos: _devices];

	[self.tableView reloadData];
}
	

#pragma mark -
#pragma mark MBProgressHUDDelegate methods
- (void)hudWasHidden {
	// Remove HUD from screen when the HUD was hidden
    [_hud removeFromSuperview];
    [_hud release];
	_hud = nil;
	
	[self discoverDevicesCompletion];
}


#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    self.clearsSelectionOnViewWillAppear = NO;
    self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/
/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/

// Ensure that the view controller supports rotation and that the split view can therefore show in both portrait and landscape.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}


#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)aTableView {
    // Return the number of sections.
    return 1;
}


- (NSInteger)tableView:(UITableView *)aTableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
	int nrDevices = 0;
	if (_devices == nil) {
		nrDevices = 0;
	} else {
		nrDevices = [_devices count];
	}
	
	if (nrDevices > 0) {
		return nrDevices;
	} else {
		return 1; // 1 line for the "No devices found" message
	}
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"CellIdentifier";
    
    // Dequeue or create a cell of the appropriate type.
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    // Configure the cell.
    //cell.textLabel.text = ;
	int rowNr = indexPath.row;
	NSString *deviceInfoString = @"No devices found.";
	if (_discoveringDevices) {
		deviceInfoString = @"Discovering devices...";
	}
	if (_devices != nil) {
		if (indexPath.row < [_devices count]) {
			UpnpDevice *device = [_devices objectAtIndex:rowNr];
			deviceInfoString = [[device _deviceInfo] friendlyName];
			if (deviceInfoString == nil) {
				NSURL *url = device._url;
				deviceInfoString = [url absoluteString];
			}
		}
	}
	cell.textLabel.text = deviceInfoString;
	
    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:YES];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)aTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    /*
     When a row is selected, set the detail view controller's detail item to the item associated with the selected row.
     */
	int rowNr = indexPath.row;
	if ((_devices == nil) ||
		([_devices count] <= rowNr)){
		_detailViewController.detailItem = [NSString stringWithFormat:@"No detail information available"];
	} else {
		_detailViewController.detailItem = [_devices objectAtIndex: indexPath.row];
	}
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
	[_upnpManager release];
    [_detailViewController release];
    [super dealloc];
}


@end

